using System.Collections.Generic;
using System.IO;
using System.Net;
using System.Net.Http;
using System.Text;
using System.Text.Json;
using System.Threading.Tasks;

using TestProject.Models;

namespace TestProject
{
    /// <summary>
    ///  Ŭ̾Ʈ
    /// </summary>
    public class TodoClient
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// JSON ȭ ɼ
        /// </summary>
        private static readonly JsonSerializerOptions _jsonSerializerOptions = new JsonSerializerOptions
        {
            IgnoreNullValues     = true,
            PropertyNamingPolicy = JsonNamingPolicy.CamelCase
        };

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Instance
        //////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// HTTP Ŭ̾Ʈ
        /// </summary>
        private readonly HttpClient httpClient;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region  - TodoClient(httpClient)

        /// <summary>
        /// 
        /// </summary>
        /// <param name="httpClient">HTTP Ŭ̾Ʈ</param>
        public TodoClient(HttpClient httpClient)
        {
            this.httpClient = httpClient;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Metohd
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region Ʈ ϱ (񵿱) - GetListAsync()

        /// <summary>
        /// Ʈ ϱ (񵿱)
        /// </summary>
        /// <returns> Ʈ ½ũ</returns>
        public async Task<List<TodoModel>> GetListAsync()
        {
            using HttpResponseMessage httpResponseMessage = await this.httpClient.GetAsync("/api/todo");

            httpResponseMessage.EnsureSuccessStatusCode();

            using Stream stream = await httpResponseMessage.Content.ReadAsStreamAsync();

            return await JsonSerializer.DeserializeAsync<List<TodoModel>>(stream, _jsonSerializerOptions);
        }

        #endregion
        #region ׸ ϱ (񵿱) - GetItemAsync(id)

        /// <summary>
        /// ׸ ϱ (񵿱)
        /// </summary>
        /// <param name="id">ID</param>
        /// <returns> ½ũ</returns>
        public async Task<TodoModel> GetItemAsync(long id)
        {
            using HttpResponseMessage httpResponseMessage = await this.httpClient.GetAsync($"/api/todo/{id}");

            if(httpResponseMessage.StatusCode == HttpStatusCode.NotFound)
            {
                return null;
            }

            httpResponseMessage.EnsureSuccessStatusCode();

            using Stream stream = await httpResponseMessage.Content.ReadAsStreamAsync();

            return await JsonSerializer.DeserializeAsync<TodoModel>(stream, _jsonSerializerOptions);
        }

        #endregion
        #region ׸ ϱ (񵿱) - CreateItemAsync(todo)

        /// <summary>
        /// ׸ ϱ (񵿱)
        /// </summary>
        /// <param name="todo"></param>
        /// <returns>½ũ</returns>
        public async Task CreateItemAsync(TodoModel todo)
        {
            StringContent stringContent = new StringContent
            (
                JsonSerializer.Serialize(todo, _jsonSerializerOptions),
                Encoding.UTF8,
                "application/json"
            );

            using HttpResponseMessage httpResponseMessage = await this.httpClient.PostAsync
            (
                "/api/todo",
                stringContent
            );

            httpResponseMessage.EnsureSuccessStatusCode();
        }

        #endregion
        #region ׸ ϱ (񵿱) - SaveItemAsync(todo)

        /// <summary>
        /// ׸ ϱ (񵿱)
        /// </summary>
        /// <param name="todo"></param>
        /// <returns>½ũ</returns>
        public async Task SaveItemAsync(TodoModel todo)
        {
            StringContent stringContent = new StringContent
            (
                JsonSerializer.Serialize(todo),
                Encoding.UTF8,
                "application/json"
            );

            using HttpResponseMessage httpResponseMessage = await this.httpClient.PutAsync
            (
                $"/api/todo/{todo.ID}",
                stringContent
            );

            httpResponseMessage.EnsureSuccessStatusCode();
        }

        #endregion
        #region ׸ ϱ (񵿱) - DeleteItemAsync(id)

        /// <summary>
        /// ׸ ϱ (񵿱)
        /// </summary>
        /// <param name="id">ID</param>
        /// <returns>½ũ</returns>
        public async Task DeleteItemAsync(long id)
        {
            using HttpResponseMessage httpResponseMessage = await this.httpClient.DeleteAsync($"/api/todo/{id}");

            httpResponseMessage.EnsureSuccessStatusCode();
        }

        #endregion
    }
}